Index: src/http/modules/ngx_http_image_filter_module.c
===================================================================
--- src/http/modules/ngx_http_image_filter_module.c	(revision 4612)
+++ src/http/modules/ngx_http_image_filter_module.c	(working copy)
@@ -40,6 +40,8 @@
     ngx_uint_t                   filter;
     ngx_uint_t                   width;
     ngx_uint_t                   height;
+    ngx_uint_t                   crop_x_offset;
+    ngx_uint_t                   crop_y_offset;
     ngx_uint_t                   angle;
     ngx_uint_t                   jpeg_quality;
     ngx_uint_t                   sharpen;
@@ -48,6 +50,8 @@
 
     ngx_http_complex_value_t    *wcv;
     ngx_http_complex_value_t    *hcv;
+    ngx_http_complex_value_t    *cxocv;
+    ngx_http_complex_value_t    *cyocv;
     ngx_http_complex_value_t    *acv;
     ngx_http_complex_value_t    *jqcv;
     ngx_http_complex_value_t    *shcv;
@@ -66,6 +70,8 @@
     ngx_uint_t                   height;
     ngx_uint_t                   max_width;
     ngx_uint_t                   max_height;
+    ngx_uint_t                   crop_x_offset;
+    ngx_uint_t                   crop_y_offset;
     ngx_uint_t                   angle;
 
     ngx_uint_t                   phase;
@@ -108,6 +114,8 @@
     void *conf);
 static char *ngx_http_image_filter_jpeg_quality(ngx_conf_t *cf,
     ngx_command_t *cmd, void *conf);
+static char *ngx_http_image_filter_crop_offset(ngx_conf_t *cf,
+    ngx_command_t *cmd, void *conf);
 static char *ngx_http_image_filter_sharpen(ngx_conf_t *cf, ngx_command_t *cmd,
     void *conf);
 static ngx_int_t ngx_http_image_filter_init(ngx_conf_t *cf);
@@ -129,6 +137,13 @@
       0,
       NULL },
 
+    { ngx_string("image_filter_crop_offset"),
+      NGX_HTTP_LOC_CONF|NGX_CONF_TAKE2,
+      ngx_http_image_filter_crop_offset,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      0,
+      NULL },
+
     { ngx_string("image_filter_sharpen"),
       NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
       ngx_http_image_filter_sharpen,
@@ -529,6 +544,20 @@
         return NULL;
     }
 
+    ctx->crop_x_offset = ngx_http_image_filter_get_value(r, conf->cxocv,
+                                                    conf->crop_x_offset);
+
+    if (ctx->crop_x_offset > 100) {
+        return NULL;
+    }
+
+    ctx->crop_y_offset = ngx_http_image_filter_get_value(r, conf->cyocv,
+                                                    conf->crop_y_offset);
+
+    if (ctx->crop_y_offset > 100) {
+        return NULL;
+    }
+
     if (rc == NGX_OK
         && ctx->width <= ctx->max_width
         && ctx->height <= ctx->max_height
@@ -737,7 +766,7 @@
 {
     int                            sx, sy, dx, dy, ox, oy, ax, ay, size,
                                    colors, palette, transparent, sharpen,
-                                   red, green, blue, t;
+                                   red, green, blue, t, left_ox, top_oy;
     u_char                        *out;
     ngx_buf_t                     *b;
     ngx_uint_t                     resize;
@@ -932,8 +961,8 @@
                 return NULL;
             }
 
-            ox /= 2;
-            oy /= 2;
+            left_ox = ox * ctx->crop_x_offset / 100;
+            top_oy = oy * ctx->crop_y_offset / 100;
 
             ngx_log_debug4(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
                            "image crop: %d x %d @ %d x %d",
@@ -944,7 +973,7 @@
                 gdImageAlphaBlending(dst, 0);
             }
 
-            gdImageCopy(dst, src, 0, 0, ox, oy, dx - ox, dy - oy);
+            gdImageCopy(dst, src, 0, 0, left_ox, top_oy, dx - ox + left_ox, dy - oy + top_oy);
 
             if (colors) {
                 gdImageTrueColorToPalette(dst, 1, 256);
@@ -1171,6 +1200,8 @@
 
     conf->filter = NGX_CONF_UNSET_UINT;
     conf->jpeg_quality = NGX_CONF_UNSET_UINT;
+    conf->crop_x_offset = NGX_CONF_UNSET_UINT;
+    conf->crop_y_offset = NGX_CONF_UNSET_UINT;
     conf->sharpen = NGX_CONF_UNSET_UINT;
     conf->angle = NGX_CONF_UNSET_UINT;
     conf->transparency = NGX_CONF_UNSET;
@@ -1207,6 +1238,18 @@
         conf->jqcv = prev->jqcv;
     }
 
+    /* 50 is ox and oy default crop offset */
+    ngx_conf_merge_uint_value(conf->crop_x_offset, prev->crop_x_offset, 50);
+    ngx_conf_merge_uint_value(conf->crop_y_offset, prev->crop_y_offset, 50);
+
+    if (conf->cxocv == NULL) {
+        conf->cxocv = prev->cxocv;
+    }
+
+    if (conf->cyocv == NULL) {
+        conf->cyocv = prev->cyocv;
+    }
+
     ngx_conf_merge_uint_value(conf->sharpen, prev->sharpen, 0);
 
     if (conf->shcv == NULL) {
@@ -1428,6 +1471,83 @@
 
 
 static char *
+ngx_http_image_filter_crop_offset(ngx_conf_t *cf, ngx_command_t *cmd,
+    void *conf)
+{
+    ngx_http_image_filter_conf_t *imcf = conf;
+
+    ngx_str_t                         *value;
+    ngx_int_t                          n;
+    ngx_http_complex_value_t           cv;
+    ngx_http_compile_complex_value_t   ccv;
+
+    value = cf->args->elts;
+
+    ngx_memzero(&ccv, sizeof(ngx_http_compile_complex_value_t));
+
+    ccv.cf = cf;
+    ccv.value = &value[1];
+    ccv.complex_value = &cv;
+
+    if (ngx_http_compile_complex_value(&ccv) != NGX_OK) {
+        return NGX_CONF_ERROR;
+    }
+
+    if (cv.lengths == NULL) {
+        n = ngx_http_image_filter_value(&value[1]);
+
+        if (n < 0) {
+            ngx_conf_log_error(NGX_LOG_EMERG, cf, 0,
+                               "invalid value \"%V\"", &value[1]);
+            return NGX_CONF_ERROR;
+        }
+
+        imcf->crop_x_offset = (ngx_uint_t) n;
+
+    } else {
+        imcf->cxocv = ngx_palloc(cf->pool, sizeof(ngx_http_complex_value_t));
+        if (imcf->cxocv == NULL) {
+            return NGX_CONF_ERROR;
+        }
+
+        *imcf->cxocv = cv;
+    }
+
+    ngx_memzero(&ccv, sizeof(ngx_http_compile_complex_value_t));
+
+    ccv.cf = cf;
+    ccv.value = &value[2];
+    ccv.complex_value = &cv;
+
+    if (ngx_http_compile_complex_value(&ccv) != NGX_OK) {
+        return NGX_CONF_ERROR;
+    }
+
+    if (cv.lengths == NULL) {
+        n = ngx_http_image_filter_value(&value[2]);
+
+        if (n < 0) {
+            ngx_conf_log_error(NGX_LOG_EMERG, cf, 0,
+                               "invalid value \"%V\"", &value[2]);
+            return NGX_CONF_ERROR;
+        }
+
+        imcf->crop_y_offset = (ngx_uint_t) n;
+
+    } else {
+        imcf->cyocv = ngx_palloc(cf->pool, sizeof(ngx_http_complex_value_t));
+        if (imcf->cyocv == NULL) {
+            return NGX_CONF_ERROR;
+        }
+
+        *imcf->cyocv = cv;
+    }
+
+    return NGX_CONF_OK;
+}
+
+
+static char *
 ngx_http_image_filter_sharpen(ngx_conf_t *cf, ngx_command_t *cmd,
     void *conf)
 {